var CommandParser = (function() {
    var parse = function(str, lookForQuotes) {
        var args = [];
        var readingPart = false;
        var part = '';
        for(var i=0; i<str.length; i++) {
            if(str.charAt(i) === ' ' && !readingPart) {
                args.push(part);
                part = '';
            } else {
                if(str.charAt(i) === '\"' && lookForQuotes) {
                    readingPart = !readingPart;
                } else {
                    part += str.charAt(i);
                }
            }
        }
        args.push(part);
        return args;
    }
    return {
        parse: parse
    }
})();
var Peppermint = function()
{
	//var editor; 
	var self = this;
	this.documentStates = {};

	/*********************************
	 Initialisation
	 *********************************/

	// Main setup
	
	this.editor = ace.edit("editor");
	this.editor.setTheme("ace/theme/peppermint");
	this.editor.getSession().setMode("ace/mode/php");
	this.editor.setOptions({
		enableBasicAutocompletion: true,
		enableSnippets: true,
		enableLiveAutocompletion: true,
		enableEmmet: true
	});

	// Attach event observers

	this.editor.session.on("change", function(){
		window.__.documentChanged();
		$Events.exec("documentChanged");
	});

	this.editor.selection.on("changeCursor", function(e) { 
		var cursor = self.editor.selection.getCursor();

		window.__.cursorChanged();
		$Events.exec("cursorChanged");

		if (typeof core === 'undefined') console.log(cursor);
		else core.currentLocationChanged_("Line " + (cursor.row+1) + ", Column " + (cursor.column+1));
	});

	this.editor.on("guttermousedown", function(e){ 
		var target = e.domEvent.target; 
		if (target.className.indexOf("ace_gutter-cell") == -1) return; 
		if (!self.editor.isFocused()) return; 
		if (e.clientX > 25 + target.getBoundingClientRect().left) return; 

		var row = e.getDocumentPosition().row;

		if (self.editor.session.$breakpoints[row]=="ace_breakpoint")
			self.bookmarkUnmarkLine(row);
		else
			self.bookmarkMarkLine(row);

		e.stop();
	}); 
	
	// UNbind keys
	
	this.unbindKey = function(key) {
	    self.editor.commands.bindKey(key,null);
	}

	this.getItems = function() {
		var TokenIterator = ace.require("ace/token_iterator").TokenIterator;
	    var iterator = new TokenIterator(pp.editor.getSession(), 0, 0);
	    var token = iterator.getCurrentToken();
	    
	    var func = '';
	    var matches = [];
	    while (token) {
	    	if( 
	    		( token.type == 'keyword' && token.value == 'function' ) || //php function
	    		( token.type == 'storage.type' && token.value == 'function' ) //js function
	    	){
	    		func = token.value;
	    	}else if( func && token.type == 'paren.lparen' && token.value == '{' ){ //stop when we get to curly bracket
	    		matches.push(func);
	    		func = '';
	    	}else if( func ){
	    		func += token.value;
	    	}else{
	    		func = '';
	    	}
	    
	    	token = iterator.stepForward();
	    }
	    
	    matches.sort();
	    	
	    return matches;
	}
	
	this.unbindKey("Cmd-N");        // File - New
	this.unbindKey("Cmd-O");        // File - Open
	this.unbindKey("Cmd-S");        // File - Save
	this.unbindKey("Cmd-Shift-S");  // File - Save As
	this.unbindKey("Cmd-P");        // File - Print
	
	//this.unbindKey("Cmd-Z");        // Edit - Undo
	//this.unbindKey("Cmd-Shift-Z");  // Edit - Redo
	this.unbindKey("Cmd-X");        // Edit - Cut
	this.unbindKey("Cmd-C");        // Edit - Copy
	this.unbindKey("Cmd-V");        // Edit - Paste
	this.unbindKey("Cmd-Shift-W");  // Edit - Select - Word
	this.unbindKey("Cmd-Shift-L");  // Edit - Select - Line
	this.unbindKey("Cmd-]");        // Edit - Transform - Indent
	this.unbindKey("Cmd-[");        // Edit - Transform - Outdent
	this.unbindKey("Ctrl-C");       // Edit - Comment/Uncomment
	
	this.unbindKey("Cmd-F");        // Find - Find
	this.unbindKey("Cmd-G");        // Find - Find Next
	this.unbindKey("Cmd-Shift-G");  // Find - Find Previous
	this.unbindKey("Alt-Shift-F");  // Find - Replace
	this.unbindKey("Ctrl-Shift-F"); // Find - Replace All
	this.unbindKey("Cmd-E");        // Find - Use selection...
	
	this.unbindKey("Ctrl-Cmd-S"); // View - Sidebar
	this.unbindKey("Ctrl-Cmd-T"); // View - Tabs
	this.unbindKey("Ctrl-Cmd-B"); // View - Status Bar
	this.unbindKey("Ctrl-Cmd-P"); // View - Panel
	this.unbindKey("Ctrl-Cmd-U"); // View - Utility View
	this.unbindKey("Ctrl-Cmd-G"); // View - Gutter
	this.unbindKey("Ctrl-Cmd-M"); // View - Print Margin
	this.unbindKey("Ctrl-Cmd-I"); // View - Indentation Guides
	this.unbindKey("Ctrl-Cmd-V"); // View - Invisibles
	this.unbindKey("Ctrl-Cmd-W"); // View - Use Word Wrap
	this.unbindKey("Cmd-=");      // View - Font - Zoom In
	this.unbindKey("Cmd--");      // View - Font - Zoom Out
	
	this.unbindKey("Ctrl-G");       // Document - Go to Line
	this.unbindKey("Ctrl-B");       // Document - Bookmark current
	this.unbindKey("Ctrl-Shift-B"); // Document - Unbookmark current
	//this.unbindKey("Ctrl-F");       // Document - Fold current
	//this.unbindKey("Ctrl-Shift-F"); // Document - Unfold current
	
	this.unbindKey("Cmd-R");        // Plugin - Run
	this.unbindKey("Cmd-Shift-R");  // Plugin - Run with Arguments
	
	this.unbindKey("Cmd-M");        // Window - Minimize
	
	this.unbindKey("Cmd-,");        // MAIN - Preferences
	this.unbindKey("Cmd-Q");        // MAIN - Quit
	
	/*********************************
	 Interface
	 *********************************/

	// Commands

	this.exec = function(what,display) {
		result = core.exec_(JSON.stringify(what));

		if (typeof display == 'undefined') terminal.echo(result);

		return result;
	}

	this.execWithEnv = function(what,env,display) {
		result = core.exec_env_(JSON.stringify(what),JSON.stringify(env));

		if (typeof display == 'undefined') terminal.echo(result);

		return result;
	}
    
    this.execApplescript = function(path,display){
        result = core.execApplescript_(path);
        
        if (typeof display == 'undefined') terminal.echo(result);
        
        return result;
    }

	this.execShell = function(what) {
		terminal.echo("shell> "+what);
		
	}

	this.execRuby = function(script,prm,callback) {
		cmd = "/usr/bin/env";

		arguments = [ "ruby", script, prm ];

		core.execute_args_callback_(cmd, JSON.stringify(arguments), callback);
	}

	this.execPHP = function(script,args,callback) {
		cmd = "/usr/bin/env";

		arguments = [ "php", script ];

		for (var arg in args)
		{
			arguments.push(arg);
		}

		core.execute_args_callback_(cmd, JSON.stringify(arguments), callback);
	}

	this.execPython = function(script,args,callback) {
		cmd = "/usr/bin/env";

		arguments = [ "python", script ];

		for (var arg in args)
			arguments.push(arg);

		core.execute_args_callback_(cmd, JSON.stringify(arguments), callback);
	}

	this.echo = function(message) {
		if ((message!=null) && (message!="(null)"))
		{
			var msg_lines = message.split("\n");

			for (var m=0; m<msg_lines.length; m++)
			{
				if (msg_lines[m].trim() != "")
					terminal.echo(" % " + msg_lines[m]);
				else
					terminal.echo("");
			}
		}
	}

	this.getFullPath = function()
	{
		return core.getFullPath();
	}

	this.setContentFromJson = function(json)
	{
		var content = eval(json)[0];
		self.editor.getSession().setValue(content);
	}

	this.setContent = function(content) {
		self.editor.getSession().setValue(content);
	}

	this.setUndoableContent = function(content) {
		self.editor.setValue(content);
	}

	this.getContent = function() {
		return self.editor.getSession().getValue();
	}

	this.setMode = function(mode) {
		self.editor.getSession().setMode("ace/mode/"+mode);
	}

	this.getMode = function() {
		return self.editor.getSession().getMode().$id.replace("ace/mode/","");
	}

	this.setCursor = function(column,row) {
		self.editor.moveCursorToPosition({row:row, column:column});
	}

	this.getCursor = function() {
		return self.editor.selection.getCursor();
	}

	this.togglePanel = function (onoff) {
		if (onoff) mainLayout.show("south");
		else mainLayout.hide("south");
	}

	this.toggleGutter = function (onoff) {
		self.editor.renderer.setShowGutter(onoff);
	}

	this.togglePrintMargin = function (onoff) {
		self.editor.setShowPrintMargin(onoff);
	}

	this.toggleIndentationGuides = function (onoff) {
		self.editor.renderer.setDisplayIndentGuides(onoff);
	}

	this.toggleInvisibles = function (onoff) {
		self.editor.setShowInvisibles(onoff);
	}

	this.toggleSoftTabs = function (onoff) {
		self.editor.setUseSoftTabs(onoff);
	}

	this.toggleWordWrap = function (onoff) {
		self.editor.getSession().setUseWrapMode(onoff);
	}

	this.setTabSize = function (n) {
		self.editor.getSession().setTabSize(n);
	}

	this.setWrapLimit = function (n) {
		self.editor.getSession().setWrapLimitRange(n,n);
	}

	this.setFontSize = function (n) {
		$("#editor").css("font-size",n+"px");
	}

	this.setTheme = function (t) {
		self.editor.setTheme("ace/theme/"+t);
	}

	this.find = function () {
		var initialPos = pp.getCursor();

		var total = self.editor.findAll( 
			$("#search-input").val(),
			{
				backwards: false,
				wrap: true,
				regExp: $("#search-opt-regex").hasClass("active"),
				caseSensitive: $("#search-opt-casesensitive").hasClass("active"),
				wholeWord: $("#search-opt-wholeword").hasClass("active")
			}
		);

		pp.setCursor(initialPos.column, initialPos.row);

		self.editor.find( 
			$("#search-input").val(),
			{
				backwards: false,
				wrap: true,
				regExp: $("#search-opt-regex").hasClass("active"),
				caseSensitive: $("#search-opt-casesensitive").hasClass("active"),
				wholeWord: $("#search-opt-wholeword").hasClass("active")
			}
		);

		if (total>0)
			$("#matches-found").html(total + " matches found<!--&nbsp;<a href='#'><i class='glyphicon glyphicon-chevron-up'></i></a>-->");
		else
			$("#matches-found").html("Unable to find any match");

		return total;
	}

	this.findNext = function () {
		self.editor.findNext();
	}

	this.findPrevious = function () {
		self.editor.findPrevious();
	}

	this.useSelectionForFind = function() {
		$("#search-input").val(self.getSelection());
	}

	this.replace = function () {
		self.editor.replace(
			$("#replace-input").val(),
			{
				backwards: false,
				wrap: true,
				regExp: $("#search-opt-regex").hasClass("active"),
				caseSensitive: $("#search-opt-casesensitive").hasClass("active"),
				wholeWord: $("#search-opt-wholeword").hasClass("active")
			}
		);
	}

	this.replaceAll = function () {
		self.editor.replaceAll(
			$("#replace-input").val(),
			{
				backwards: false,
				wrap: true,
				regExp: $("#search-opt-regex").hasClass("active"),
				caseSensitive: $("#search-opt-casesensitive").hasClass("active"),
				wholeWord: $("#search-opt-wholeword").hasClass("active")
			}
		);
	}

	this.scrollPageUp = function() {
		self.editor.scrollPageUp();
	}

	this.scrollPageDown = function() {
		self.editor.scrollPageDown();
	}

	this.scrollToSelection = function() {
		self.editor.renderer.scrollSelectionIntoView();
	}

	this.bookmarkMarkLine = function(line) {
		self.editor.session.setBreakpoint(line,'ace_breakpoint');
	}

	this.bookmarkMarkCurrent = function() {
		var curs = self.editor.selection.getCursor();
		self.bookmarkMarkLine(curs.row);
	}

	this.bookmarkUnmarkLine = function(line) {
		self.editor.session.clearBreakpoint(line);
	}

	this.bookmarkUnmarkCurrent = function() {
		var curs = self.editor.selection.getCursor();
		self.bookmarkUnmarkLine(curs.row);
	}

	this.bookmarkRemoveAll = function() {
		self.editor.session.clearBreakpoints();
	}

	this.foldAll = function() {
		self.editor.getSession().foldAll();
	}

	this.unfoldAll = function() {
		self.editor.getSession().expandFolds(self.editor.getSession().getAllFolds());
	}

	this.foldCurrent = function() {
		self.editor.getSession().toggleFold(false);
	}

	this.unfoldCurrent = function() {
		self.editor.getSession().toggleFold(true);
	}

	this.toggleComments = function() {
		self.editor.toggleCommentLines();
	}

	this.goToLine = function(line) {
		self.editor.gotoLine(line);
	}

	this.selectWord = function() {
		self.editor.selection.selectWord();
	}

	this.selectLine = function() {
		self.editor.selection.selectLine();
	}

	this.selectLineEnd = function() {
		self.editor.selection.selectLineEnd();
	}

	this.selectLineStart = function() {
		self.editor.selection.selectLineStart();
	}

	this.selectFileStart = function() {
		self.editor.selection.selectFileStart();
	}

	this.selectFileEnd = function() {
		self.editor.selection.selectFileEnd();
	}

	this.selectAll = function() {
		self.editor.selectAll();
	}

	this.deselectAll = function() {
		self.editor.clearSelection();
	}

	this.indent = function() {
		self.editor.blockIndent();
	}

	this.outdent = function() {
		self.editor.blockOutdent();
	}

	this.moveUp = function() {
		self.editor.moveLinesUp();
	}

	this.moveDown = function() {
		self.editor.moveLinesDown();
	}

	this.noOfUndoItems = function() {
		self.editor.getSession().getUndoManager().$undoStack.length;
	}

	this.noOfRedoItems = function() {
		self.editor.getSession().getUndoManager().$redoStack.length;
	}

	this.resizeEditor = function() {
		self.editor.resize();
	}

	this.undo = function() {
		self.editor.undo();
	}

	this.redo = function() {
		self.editor.redo();
	}

	this.getSelection = function() {
		return self.editor.session.getTextRange(self.editor.getSelectionRange());
	}

	this.setSelection = function(replacement) {
		self.editor.getSession().replace(self.editor.getSelectionRange(),replacement);
	}

	this.getNoOfLines = function() {
		return self.editor.session.getLength();
	}

	this.getWordAtLocation = function() {
		return self.editor.session.getTextRange(self.editor.session.getWordRange(self.editor.selection.getCursor().row,self.editor.selection.getCursor().column));
	}

	this.getTempFileWithExtension = function(ext) {
		return core.getTempFile_withExtension_(JSON.stringify([self.getContent()]), ext);
	}

	this.doRun = function() {
		// call Obj-C to invoke "Run"
	}

	// State handling

	function changer() {
		core.currentDocumentChanged();
		window.__.documentChanged();
		
	}

	this.getFoldsState = function() {
		return self.editor.getSession().getAllFolds();
	}

	this.setFoldsState = function(foldsState) {
		self.editor.getSession().addFolds(foldsState);
	}

	this.getSelectionState = function() {
		return self.editor.getSession().selection.getRange();
	}

	this.setSelectionState = function(selectionState) {
		self.editor.getSession().selection.setRange(selectionState);
	}

	this.getBreakpointsState = function() {
		return self.editor.getSession().getBreakpoints();
	}

	this.setBreakpointsState = function(breakpointsState) {
		var indices = [];
		for (var i=0; i<breakpointsState.length; i++)
		{
			if (breakpointsState[i] === "ace_breakpoint")
				indices.push(i);
		}

		self.editor.getSession().setBreakpoints(indices);
	}

	this.getAnnotationsState = function() {
		return self.editor.getSession().getAnnotations();
	}

	this.setAnnotationsState = function(annotationsState) {
		self.editor.getSession().setAnnotations(annotationsState);
	}

	this.getUndoState = function() {
		return self.editor.getSession().getUndoManager().$undoStack;
	}

	this.setUndoState = function(undoState) {
		self.editor.getSession().getUndoManager().$undoStack = undoState;
	}

	this.getRedoState = function() {
		return self.editor.getSession().getUndoManager().$redoStack;
	}

	this.setRedoState = function(redoState) {
		self.editor.getSession().getUndoManager().$redoStack = redoState;
	}

	this.saveLocalState = function(docpointer) {
		self.documentStates[docpointer] = {};

		self.documentStates[docpointer].folds = self.getFoldsState();
		self.documentStates[docpointer].selection = self.getSelectionState();
		self.documentStates[docpointer].breakpoints = self.getBreakpointsState();
		self.documentStates[docpointer].annotations = self.getAnnotationsState();
		self.documentStates[docpointer].undo = self.getUndoState();
		self.documentStates[docpointer].redo = self.getRedoState();
	}

	this.loadLocalState = function(docpointer) {
		if (typeof this.documentStates[docpointer] !== 'undefined')
		{
			var st = this.documentStates[docpointer];

			self.setFoldsState(st.folds);
			self.setSelectionState(st.selection);
			self.setBreakpointsState(st.breakpoints);
			self.setAnnotationsState(st.annotations);
			self.setUndoState(st.undo);
			self.setRedoState(st.redo);
		}
	}

	this.setState = function(state) {
		this.setContent(state.content);
		this.setMode(state.mode);
        
        this.editor.setReadOnly(state.readOnly);

		this.editor.session.on("change", changer);

		//this.editor.focus();
	}

	this.getState = function() {
		var result = {
			content : this.getContent(),
			mode 	: this.getMode()
		};

		this.editor.getSession().removeListener('change', changer);

		//this.editor.getSession().removeAllListeners("change");

		return JSON.stringify(result);
	}

	/*********************************
	 Bindings
	 *********************************/

	$("#search-input").on("input",self.find);
	$("#search-left").click(self.findPrevious);
	$("#search-right").click(self.findNext);

	$("#replace-input").bind('keypress', function(e) 
	{
		var code = e.keyCode || e.which;
		if (code==13)
		{
			self.replace();
		}
	});

	$("#replace-one").click(self.replace);
	$("#replace-all").click(self.replaceAll);

	$("#search-opt-regex").click(function(){
		core.searchOptChanged_withValue_("PP_Search_Regex",!$("#search-opt-regex").hasClass("active"));
	});
	$("#search-opt-casesensitive").click(function(){
		core.searchOptChanged_withValue_("PP_Search_Case",!$("#search-opt-casesensitive").hasClass("active"));
	});
	$("#search-opt-wholeword").click(function(){
		core.searchOptChanged_withValue_("PP_Search_Whole_Word",!$("#search-opt-wholeword").hasClass("active"));
	});

	this.searchOptRegexToggle = function(onoff)
	{
		if (onoff) $("#search-opt-regex").addClass('active');
		else $("#search-opt-regex").removeClass('active');
	}

	this.searchOptCaseToggle = function(onoff)
	{
		if (onoff) $("#search-opt-casesensitive").addClass('active');
		else $("#search-opt-casesensitive").removeClass('active');
	}

	this.searchOptWholeWordToggle = function(onoff)
	{
		if (onoff) $("#search-opt-wholeword").addClass('active');
		else $("#search-opt-wholeword").removeClass('active');
	}

	this.showConsoleTab = function()
	{
		$(".panel-tabs li:first a").tab("show");
	}

	this.showSearchTab = function()
	{
	    setTimeout(function(){ $("#search-input").focus(); }, 500);
		$(".panel-tabs li:last a").tab("show");
		//$("#search-input").focus();
	}

	this.getPathsForDocumentWithExtension = function(ext)
	{
		var filePath = self.getFullPath();

		if (filePath==="")
			filePath = core.getTempFile_withExtension_(self.getContent(), ext);

		var fileName = filePath.replace(/^.*[\\\/]/, '');
		var folderPath = filePath.replace(fileName, '');

		var re = /(?:\.([^.]+))?$/;

		var extension = re.exec(fileName)[1];

		return {
			path: filePath,
			filename: fileName,
			folder: folderPath,
			extension: extension,
			filenameWithoutExtension: fileName.replace("."+extension, "")
		}
	}

	this.loadImage = function(base64)
	{
		var IMG = $("#imageviewer img");

		$(IMG).attr("src","data:image/png;base64," + base64);

		$(IMG).css("margin-left", "-" + parseInt($(IMG).css("width"))/2 + "px");
		$(IMG).css("margin-top", "-" + parseInt($(IMG).css("height"))/2 + "px");
	}

	this.uuid = (function() {
	  function s4() {
	    return Math.floor((1 + Math.random()) * 0x10000)
	               .toString(16)
	               .substring(1);
	  }
	  return function() {
	    return s4() + s4() + '-' + s4() + '-' + s4() + '-' +
	           s4() + '-' + s4() + s4() + s4();
	  };
	})();
}

/*******************************************************
 *
 * Initial Setup
 *
 *******************************************************/

$(function(){

	/*********************************
	 Setup Layout
	 *********************************/

	resizeEditorOnLayoutChange = function()
	{
		pp.resizeEditor();
	}

	window.mainLayout = $("body").layout({
		fxName : "none",
		fxSpeed : "fast",
		slidable: false,
		togglerLength_closed: 0,
		spacing_open: 3,
		spacing_closed: 3,

		onresize_start: resizeEditorOnLayoutChange,
		onresize_end: resizeEditorOnLayoutChange,

		default: {
			onresize_start: resizeEditorOnLayoutChange,
			onresize_end: resizeEditorOnLayoutChange
		},

		panes: {
			liveContentResizing:   true,
			livePaneResizing: 		true
		},

		south : {
			initClosed: true,
			initHidden: true,
			onclose_end: function(){
				$(".panel-tabs li").each(function(){
					$(this).removeClass("active");
				});
				resizeEditorOnLayoutChange();
				mainLayout.hide("south");

				core.panelToggled_(false); // Notify the core, so the it updates the Options + Menus
			},
			closable: true,
			resizable: true,
			onresize_start: resizeEditorOnLayoutChange,
			onresize_end: function(e){
				resizeEditorOnLayoutChange();
				 $("#console-output").css("width",$("#console-input-group").css("width"));
			},
			onopen: function(){
				
				if ($("#console-tab").hasClass("active")) { $("#console-trigger").removeClass("active").addClass("active"); terminal.focus(); }
				if ($("#search-tab").hasClass("active")) { $("#search-trigger").removeClass("active").addClass("active"); }
				core.panelToggled_(true);
			},
			onhide_end: resizeEditorOnLayoutChange,
			size: 110,
			minSize: 110,
			maxSize: $(window).height() - 150
		}
	});

	$(window).resize(function(){
		mainLayout.south.options.maxSize = $(window).height() - 150;
		mainLayout.resizeAll();
	});

	/*********************************
	 Terminal
	 *********************************/
	 
	 window.setTerminalStdin = function(onoff){
	     if (onoff){
	         window.oldTerminalMode = window.terminalMode;
	         window.oldTerminalPrompt = window.terminalPrompt;
	         
	         window.terminalMode = "stdin";
	         window.terminalPrompt = "";
	         
	         window.terminal.set_prompt("");
	     } else {
	         window.terminalMode = window.oldTerminalMode;
	         window.terminalPrompt = window.oldTerminalPrompt;
	         
	         window.terminal.set_prompt(window.terminalPrompt);
	         
	     }
	 }

	window.terminalMode = "coffee";
	window.terminalPrompt = "[[b;orange;]coffee>] ";
	window.oldTerminaMode = "coffee";
	window.oldTerminalPrompt = "[[b;orange;]coffee>] ";
	window.terminal = $('#termem').terminal(
		function(cmd,term)
		{
			switch (cmd)
			{
				//case 'version': term.echo("Version: 1/462.1\n"); break;
				//case 'help': term.echo("23 available commands\n"); break;
				case 'help':
				    terminal.echo("coffee     - Enable CoffeeScript mode (default)");
				    terminal.echo("js         - Enable JavaScript mode");
				    terminal.echo("bash       - Enable Bash mode\n");
				    terminal.echo("run        - Run current script (if available)\n");
				    terminal.echo("quit       - Quit current mode and return to default");
				    terminal.echo("clear      - Clear terminal window\n");
				    terminal.echo("version    - Show current version");
				    terminal.echo("help       - Show this help listing\n");
				    break;
				case 'version'  : 
				    terminal.echo("[[;#A5E12C;]Peppermint][[b;#65A10C;]/"+ $Core.version + "\n\n");
				    break;
				case 'quit'     : 
				    if ((terminalMode=="bash")||(terminalMode=="js")) { oldTerminalMode = terminalMode = "coffee"; oldTerminalPrompt = terminalPrompt = "[[b;orange;]coffee>] "; term.set_prompt("[[b;orange;]coffee>] "); } break;
				case 'bash'     : 
				    if (terminalMode!="bash") { oldTerminalMode = terminalMode = "bash"; oldTerminaPrompt = terminalPrompt = "[[b;white;]bash>] "; term.set_prompt("[[b;white;]bash>] "); terminalMode = "bash";}
				    else terminal.echo("You're already in Bash mode.");
				    break;
				case 'coffee'   : 
				    if (terminalMode!="coffee") { oldTerminalMode = terminalMode = "coffee"; oldTerminaPrompt = terminalPrompt = "[[b;orange;]coffee>] "; term.set_prompt("[[b;orange;]coffee>] "); terminalMode = "coffee"; }
				    else terminal.echo("You're already in Coffee mode.");
				    break;
				case 'js'  : 
				    if (terminalMode!="js") { oldTerminalMode = terminalMode = "js"; oldTerminalPrompt = terminalPrompt = "[[b;cyan;]js>] "; term.set_prompt("[[b;cyan;]js>] "); terminalMode = "js"; }
				    else terminal.echo("You're already in JavaScript mode.");
				    break;
				case 'run' :
					if (!core.execRun()) 
						terminal.echo("Run: command not available for current spec");
					break;
				default: 
				{
					if (terminalMode=="js")
					{
						try {
							var rez=eval(cmd);
							if (rez)
							term.echo(" = " + rez + "\n");
						}
						catch (e)
						{
							term.echo("[[;red;] ! ]" + e + "\n");
						}
					}
					else if (terminalMode=="coffee")
					{
					    try {
					        var coffee = cmd;
					        var decoffee = $Core.coffee(coffee,true,false);
							var rez=eval(decoffee);
							if (rez)
							term.echo(" = " + rez + "\n");
						}
						catch (e)
						{
							term.echo("[[;red;] ! ]" + e + "\n");
						}
					}
					else if (terminalMode=="stdin")
					{
					    $Terminal.write(cmd);
					}
					else if (terminalMode=="bash")
					{
					    var params = CommandParser.parse(cmd,true);
					    $Terminal.exec.apply($Terminal,params);
					}
				}
			}
		},
		/*
		{
			version: function() { this.echo("Version: 01/462.1\n"); },
			help: function() { this.echo("23 available commands\n"); },
		    shell : function(){
		    	this.push(function(cmd,term)
				{
					pp.exec(cmd);
				});
		    }
		},*/
		{ 
			prompt: '[[b;orange;]coffee>] ', 
			greetings: "[[b;#A5E12C;]Peppermint][[b;#65A10C;]/"+ $Core.version + "]\n[[;gray;]Type 'help' for a list of available commands]\n\n",
			tabcompletion: true
		}
	);

	/*********************************
	 Panel
	 *********************************/

	$("#console-trigger").click(function(){ $(".panel-tabs li:first a").tab("show"); });
	$("#search-trigger").click(function(){ $(".panel-tabs li:last a").tab("show"); });

	$('.panel-tabs a[data-toggle="tab"]').on('show.bs.tab', function (e) {
		if (mainLayout.state.south.isClosed) mainLayout.open("south");
		console.log(e.target.hash);

		if (e.target.hash=="#console-tab") terminal.focus();
	});

	//$("#run-trigger").click(function(){
	//	pp.doRun();
	//});

	/*********************************
	 Initialise Peppermint
	 *********************************/

	 window.pp = new Peppermint();
	 window.binariesIndex = JSON.parse(core.getBinariesIndex());
});;